#!/usr/bin/python
#
# check_unifi - Nagios plugin to check Unifi Controller
#
# Based on UniFi WiFi Nagios Monitoring Plugin by Michel Greijmans
# Rewritten 2016/08/26 by Michael Sweetser (mike@sweetser.info)
# Adapted for check_mk 2017/08/17 by Alexandre Gauvrit (agauvrit@tranquil.it)
#
# Requires pyunifi pip module

import argparse
import sys
from pyunifi.controller import Controller

"""check_mk plugin to check the UniFi Controller for AP status."""

# Get the arguments
parser = argparse.ArgumentParser()
parser.add_argument(
    '-H', '--hostname', default='unifi',
    help='the controller address (default "unifi")')
parser.add_argument(
    '-u', '--username', default='admin',
    help='the controller username (default("admin")')
parser.add_argument(
    '-p', '--password', default='',
    help='the controller password')
parser.add_argument(
    '-b', '--port', default='8443',
    help='the controller port (default "8443")')
parser.add_argument(
    '-v', '--version', default='v2',
    help='the controller base version (default "v2")')
parser.add_argument(
    '-s', '--siteid', default='default',
    help='the site ID, UniFi >=3.x only (default "default")')
parser.add_argument(
    '-a', '--ap', default='',
    help='name of AP to get status of (default: overall check)')
parser.add_argument(
    '-V','--no-ssl-verify', default=False, action='store_true', help='Don\'t verify ssl certificates')
args = parser.parse_args()

# Connect to the Controller (if we can)
try:
    c = Controller(args.hostname, args.username, args.password,
                   args.port, args.version, args.siteid, ssl_verify=(not args.no_ssl_verify))
except Exception as e:
    print ("UNKNOWN: Error connecting to UniFi Controller (%s)|"
           "Online=0 Offline=0" % e)
    sys.exit(1)

aps = c.get_aps()
connected_users = c.get_clients()
sites = c.get_sites()


ap_online = 0
ap_offline = 0
offline_names = []
offline_list = ""
online_names = []
online_list = ""
ap_state = 0
ap_message = ""
user_count= 0
sitename = "Unifi"
ap_count = len(aps)


if args.siteid != 'default':
    for sites_iter in sites:
        if args.siteid == sites_iter['name']:
            sitename = 'Unifi-'+sites_iter['desc']


for us in connected_users:
    if us['oui']:
        user_count +=1

# Loop through the discovered APs
for ap in aps:
    if ap['state'] == 1:
        ap_online += 1
	online_names.append(ap['name'])
    else:
        ap_offline += 1
        offline_names.append(ap['name'])

offline_names.sort()
online_names.sort()

if ap_online == 0:
    offline_list = " (%s)" % ",".join(offline_names)
    print ("2 %s online=0;0;1|offline=%d CRITICAL - No AP reachable 0/%d %s" % (sitename,ap_offline,ap_count,offline_list))
    sys.exit(1)

if ap_offline > 0:
    offline_list = " (%s)" % ",".join(offline_names)

if ap_online > 0:
    online_list = " (%s)" % ",".join(online_names)

message = ("online APs: %d/%d %s" % (ap_online,ap_count,online_list))

if ap_offline > 0 and ap_online < (ap_count*0.8):
        print ("1 %s online=%d;0;1|offline=%d %s, online clients : %d" % (sitename,ap_online, ap_offline,message,user_count))
else:
        print ("0 %s online=%d;0;1|offline=%d %s, online clients : %d" % (sitename,ap_online, ap_offline,message,user_count))
